package com.reactnativevidvliveness;

import android.app.Activity;
import android.content.Intent;
import android.util.Log;
import android.graphics.Color;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.BaseActivityEventListener;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.google.gson.Gson;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import me.vidv.vidvlivenesssdk.HostLivenessActivity;
import me.vidv.vidvlivenesssdk.databinding.VidvLivenessActivityHostBinding;
import me.vidv.vidvlivenesssdk.sdk.BuilderError;
import me.vidv.vidvlivenesssdk.sdk.CapturedActions;
import me.vidv.vidvlivenesssdk.sdk.ServiceFailure;
import me.vidv.vidvlivenesssdk.sdk.Success;
import me.vidv.vidvlivenesssdk.sdk.UserExit;
import me.vidv.vidvlivenesssdk.sdk.VIDVDetectedFace;
import me.vidv.vidvlivenesssdk.sdk.VIDVLivenessConfig;
import me.vidv.vidvlivenesssdk.sdk.VIDVLivenessListener;
import me.vidv.vidvlivenesssdk.sdk.VIDVLivenessResponse;

@ReactModule(name = VidvlivenessModule.NAME)
public class VidvlivenessModule extends ReactContextBaseJavaModule {
  public static final String NAME = "Vidvliveness";
  private Promise promise;
  private String PluginInitialzingError = "6001";

  private VIDVLivenessConfig.Builder livenessConfig;

  public VidvlivenessModule(ReactApplicationContext reactContext) {
    super(reactContext);
    livenessConfig = new VIDVLivenessConfig.Builder();
  }


  @Override
  @NonNull
  public String getName() {
    return NAME;
  }


  @ReactMethod
  public void startLiveness(ReadableMap params, Promise promise) {
    this.promise = promise;

    Activity activity = getCurrentActivity();
    HashMap<String, Object> inputs = params.toHashMap();
    ArrayList<VIDVDetectedFace> detectedFaces=new ArrayList<>();
    if (!inputs.containsKey("base_url")) {
      promise.reject(PluginInitialzingError, "please make sure you entered base url correctly");
    } else if (!inputs.containsKey("access_token")) {
      promise.reject(PluginInitialzingError, "please make sure you entered access token correctly");
    } else if (!inputs.containsKey("bundle_key")) {
      promise.reject(PluginInitialzingError, "please make sure you entered bundle key correctly");

    } else {
      if (!inputs.containsKey("language") | inputs.get("language").toString().equals(""))
        livenessConfig.setLanguage("en");
      else livenessConfig.setLanguage(inputs.get("language").toString());

      if (inputs.containsKey("liveness_number_of_failed_trials")) {
        livenessConfig.setFailTrials(Integer.valueOf((String) inputs.get("liveness_number_of_failed_trials")));
      }

      if (inputs.containsKey("liveness_time_per_action")) {
        livenessConfig.setInstructionTimer(Integer.valueOf((String) inputs.get("liveness_time_per_action")));
      }

      if (inputs.containsKey("liveness_number_of_instructions")) {
        livenessConfig.setNumberOfInstructions(Integer.valueOf((String) inputs.get("liveness_number_of_instructions")));
      }

      if (inputs.containsKey("facematch_ocr_transactionId") && !inputs.get("facematch_ocr_transactionId").toString().equals(""))
        livenessConfig.setFrontTransactionId(inputs.get("facematch_ocr_transactionId").toString());

      if (inputs.containsKey("facematch_image"))
        livenessConfig.setFaceMatchImage((byte[]) inputs.get("facematch_image"));


      if (inputs.containsKey("enable_smile") && !((Boolean) inputs.get("enable_smile"))) {
        livenessConfig.withoutSmile();
      }
      if (inputs.containsKey("enable_look_left") && !((Boolean) inputs.get("enable_look_left"))) {
        livenessConfig.withoutLookLeft();
      }
      if (inputs.containsKey("enable_look_right") && !((Boolean) inputs.get("enable_look_right"))) {
        livenessConfig.withoutLookRight();
      }
      if (inputs.containsKey("enable_close_eyes") && !((Boolean) inputs.get("enable_close_eyes"))) {
        livenessConfig.withoutCloseEyes();
      }

      if (inputs.containsKey("enable_voiceover") && !((Boolean) inputs.get("enable_voiceover"))) {
        livenessConfig.withoutVoiceOver();
      }

      if (inputs.containsKey("show_error_message")) {
         livenessConfig.showErrorDialogs((Boolean) inputs.get("show_error_message"));
      } 

     if (inputs.containsKey("primary_color")&& !inputs.get("primary_color").toString().equals(""))  {
       livenessConfig.setPrimaryColor(Color.parseColor((String) inputs.get("primary_color")));
     }

      if (inputs.containsKey("headers")) {
        HashMap<String, Object> map = (HashMap<String, Object>) inputs.get("headers");
        if (map.size() > 0) {
          HashMap<String, String> map2 = new HashMap<>();
          for (Map.Entry<String, Object> header : map.entrySet()) {
            map2.put(header.getKey(), header.getValue().toString());

          }
          livenessConfig.setHeaders(map2);
        }
      }

      livenessConfig.setBaseUrl(params.getString("base_url"));
      livenessConfig.setAccessToken(params.getString("access_token"));
      livenessConfig.setBundleKey(params.getString("bundle_key"));

      livenessConfig.start(activity, new VIDVLivenessListener() {

        @Override
        public void onLivenessResult(VIDVLivenessResponse response) {
          if (response instanceof Success) {

            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "SUCCESS");
              jsonObject.put("livenessResult", ((Success) response).vidvLivenessResult);
              jsonObject.put("capturedImages", detectedFaces);
            } catch (JSONException e) {
              e.printStackTrace();
            }
            String jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof BuilderError) {

            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "ERROR");
              jsonObject.put("errorCode", ((BuilderError) response).errorCode);
              jsonObject.put("errorMessage", ((BuilderError) response).errorMessage);
             
            } catch (JSONException e) {
              e.printStackTrace();
            }
            String jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof ServiceFailure) {

            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "FAILURE");
              jsonObject.put("errorCode", ((ServiceFailure) response).errorCode);
              jsonObject.put("errorMessage", ((ServiceFailure) response).errorMessage);
              jsonObject.put("livenessResult", ((ServiceFailure) response).vidvLivenessResult);
              jsonObject.put("capturedImages", detectedFaces);
            } catch (JSONException e) {
              e.printStackTrace();
            }
            String jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof UserExit) {

            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "EXIT");
              jsonObject.put("step", ((UserExit) response).step);
              jsonObject.put("livenessResult", ((UserExit) response).vidvLivenessResult);
              jsonObject.put("capturedImages", detectedFaces);
            } catch (JSONException e) {
              e.printStackTrace();
            }
            String jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          }else if (response instanceof CapturedActions){

            detectedFaces.add(((CapturedActions) response).detectedFace);

          }
        }
      });
    }
  }
}
